@@ -78,14 +78,27 @@ our $VERSION = '0.01';
 
 my %PREPROCESSORS = (
   coffee => 'Mojolicious::Plugin::AssetPack::Preprocessor::CoffeeScript',
-  css => 'Mojolicious::Plugin::AssetPack::Preprocessor::Css',
-  js => 'Mojolicious::Plugin::AssetPack::Preprocessor::JavaScript',
-  jsx => 'Mojolicious::Plugin::AssetPack::Preprocessor::Jsx',
-  less => 'Mojolicious::Plugin::AssetPack::Preprocessor::Less',
-  sass => 'Mojolicious::Plugin::AssetPack::Preprocessor::Sass',
-  scss => 'Mojolicious::Plugin::AssetPack::Preprocessor::Scss',
+  css    => 'Mojolicious::Plugin::AssetPack::Preprocessor::Css',
+  js     => 'Mojolicious::Plugin::AssetPack::Preprocessor::JavaScript',
+  jsx    => 'Mojolicious::Plugin::AssetPack::Preprocessor::Jsx',
+  less   => 'Mojolicious::Plugin::AssetPack::Preprocessor::Less',
+  sass   => 'Mojolicious::Plugin::AssetPack::Preprocessor::Sass',
+  scss   => 'Mojolicious::Plugin::AssetPack::Preprocessor::Scss',
 );
 
+=head1 ATTRIBUTES
+
+=head2 fallback
+
+TODO
+
+=cut
+
+has fallback => sub {
+  require Mojolicious::Plugin::AssetPack::Preprocessor::Fallback;
+  Mojolicious::Plugin::AssetPack::Preprocessor::Fallback->new;
+};
+
 =head1 METHODS
 
 =head2 add
@@ -151,16 +164,18 @@ and returns a combined checksum.
 =cut
 
 sub checksum {
-  my($self, $extension, $text, $filename) = @_;
+  my ($self, $extension, $text, $filename) = @_;
   my $old_dir = getcwd;
-  my $err = '';
+  my $err     = '';
   my @checksum;
 
   local $@;
 
   eval {
     chdir dirname $filename if $filename;
-    push @checksum, $_->checksum($text, $filename) for $self->_preprocessors($extension);
+    for my $p ($self->_preprocessors($extension)) {
+      push @checksum, $p->checksum($text, $filename);
+    }
     1;
   } or do {
     $err = $@ || "AssetPack failed with unknown error while processing $filename.\n";
@@ -178,8 +193,7 @@ DEPRECATED. Default handlers are added on the fly.
 =cut
 
 sub detect {
-  warn "DEPRECATED".
-  $_[0];
+  warn "DEPRECATED" . $_[0];
 }
 
 =head2 process
@@ -192,23 +206,26 @@ called with the C<$assetpack> object as the first argument.
 =cut
 
 sub process {
-  my($self, $extension, $assetpack, $text, $filename) = @_;
+  my ($self, $extension, $assetpack, $text, $filename) = @_;
   my $old_dir = getcwd;
-  my $err = '';
+  my $err     = '';
 
   local $@;
 
   eval {
     chdir dirname $filename;
-    $_->($_, $assetpack, $text, $filename) for $self->_preprocessors($extension);
+    for my $p ($self->_preprocessors($extension)) {
+      $p->($p, $assetpack, $text, $filename);
+      $assetpack->{log}->error($p->errmsg) if $p->errmsg;
+      $err ||= $p->errmsg;
+    }
     1;
   } or do {
     $err = $@ || "AssetPack failed with unknown error while processing $filename.\n";
   };
 
   chdir $old_dir;
-  die $err if $err;
-  $self;
+  return $err;
 }
 
 =head2 map_type
@@ -229,7 +246,7 @@ sub remove { shift->unsubscribe(@_) }
 
 sub _preprocessors {
   my ($self, $extension) = @_;
-  my @preprocessors = @{ $self->subscribers($extension) };
+  my @preprocessors = @{$self->subscribers($extension)};
 
   return @preprocessors if @preprocessors;
 
@@ -239,7 +256,7 @@ sub _preprocessors {
     return $self->add($extension => $class->new);
   }
 
-  return;
+  return $self->fallback;
 }
 
 =head1 AUTHOR
